Package: CohortCharacteristics
Type: Package
Title: Summarise and Visualise Characteristics of Patients in the OMOP
        CDM
Version: 1.1.3
Authors@R: c(
    person("Martí", "Català", , "marti.catalasabate@ndorms.ox.ac.uk", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3308-9905")),
    person("Yuchen", "Guo", , "yuchen.guo@ndorms.ox.ac.uk", role = c("aut"),
    comment = c(ORCID = "0000-0002-0847-4855")),
    person("Mike", "Du", , "mike.du@ndorms.ox.ac.uk", role = c("ctb"), 
    comment = c(ORCID = "0000-0002-9517-8834")),
    person("Kim", "Lopez-Guell", , "kim.lopez@spc.ox.ac.uk", role = c("aut"),
    comment = c(ORCID = "0000-0002-8462-8668")),
    person("Edward", "Burn", , "edward.burn@ndorms.ox.ac.uk", role = c("aut"),
    comment = c(ORCID = "0000-0002-9286-1128")),
    person("Nuria", "Mercade-Besora", , "nuria.mercadebesora@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0006-7948-3747")),
    person("Marta", "Alcalde", , "marta.alcaldeherraiz@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0002-4405-1814"))
    )
Maintainer: Martí Català <marti.catalasabate@ndorms.ox.ac.uk>
Description: Summarise and visualise the characteristics of patients in data 
    mapped to the Observational Medical Outcomes Partnership (OMOP) common data 
    model (CDM).
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, tidyr, rlang, cli, stringr, omopgenerics (>= 1.3.1),
        PatientProfiles (>= 1.3.1), snakecase, lifecycle, purrr, clock
URL: https://darwin-eu.github.io/CohortCharacteristics/
BugReports: https://github.com/darwin-eu/CohortCharacteristics/issues
Language: en-US
Depends: R (>= 4.1)
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
Suggests: CDMConnector (>= 1.6.0), CodelistGenerator, visOmopResults
        (>= 1.2.0), CohortConstructor, covr, DBI, DiagrammeR,
        DiagrammeRsvg, DrugUtilisation, DT, duckdb (>= 1.0.0),
        flextable, ggplot2, ggpubr, glue, grid, gt, here, Hmisc,
        htmltools, knitr, odbc, omock, plotly, png, reactable,
        rmarkdown, RPostgres, rsvg, scales, spelling, testthat (>=
        3.1.5), tictoc, withr, extrafont
NeedsCompilation: no
Packaged: 2026-05-05 22:25:33 UTC; martics
Author: Martí Català [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3308-9905>),
  Yuchen Guo [aut] (ORCID: <https://orcid.org/0000-0002-0847-4855>),
  Mike Du [ctb] (ORCID: <https://orcid.org/0000-0002-9517-8834>),
  Kim Lopez-Guell [aut] (ORCID: <https://orcid.org/0000-0002-8462-8668>),
  Edward Burn [aut] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Nuria Mercade-Besora [aut] (ORCID:
    <https://orcid.org/0009-0006-7948-3747>),
  Marta Alcalde [aut] (ORCID: <https://orcid.org/0009-0002-4405-1814>)
Repository: CRAN
Date/Publication: 2026-05-06 05:10:50 UTC
Built: R 4.5.2; ; 2026-05-06 06:44:34 UTC; unix
