---
title: "Summarise cohort overlap"
output: 
  html_document:
    pandoc_args: [
      "--number-offset=1,0"
      ]
    number_sections: yes
    toc: yes
vignette: >
  %\VignetteIndexEntry{a08_summariseCohortOverlap}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", message = FALSE, warning = FALSE
)

library(CDMConnector)
if (Sys.getenv("EUNOMIA_DATA_FOLDER") == "") Sys.setenv("EUNOMIA_DATA_FOLDER" = tempdir())
if (!dir.exists(Sys.getenv("EUNOMIA_DATA_FOLDER"))) dir.create(Sys.getenv("EUNOMIA_DATA_FOLDER"))
if (!eunomia_is_available()) downloadEunomiaData()
```
  
When creating multiple cohorts we might be interested in the overlap between them. That is, how many individuals appear in multiple cohorts. CohortCharacteristics provides functions to generate such estimates and then summarise these estimates in tables and plots.

To see how this works let's create a few medication cohorts with the Eunomia synthetic dataset.

```{r, message=FALSE, warning = FALSE}
library(CDMConnector)
library(CodelistGenerator)
library(CohortCharacteristics)
library(dplyr)
library(ggplot2)

con <- DBI::dbConnect(duckdb::duckdb(),
  dbdir = CDMConnector::eunomia_dir()
)
cdm <- CDMConnector::cdm_from_con(con,
  cdm_schem = "main",
  write_schema = "main",
  cdm_name = "Eunomia"
)

meds_cs <- getDrugIngredientCodes(
  cdm = cdm,
  name = c(
    "acetaminophen",
    "morphine",
    "warfarin"
  )
)

cdm <- generateConceptCohortSet(
  cdm = cdm,
  name = "meds",
  conceptSet = meds_cs,
  end = "event_end_date",
  limit = "all",
  overwrite = TRUE
)

settings(cdm$meds)
cohortCount(cdm$meds)
```

Now we have our cohorts we can summarise the overlap between them.

```{r}
meds_overlap <- cdm$meds |>
  summariseCohortOverlap()
meds_overlap |>
  glimpse()
```

We have table and plotting functions to help view our results.

```{r}
tableCohortOverlap(meds_overlap)
```

```{r}
plotCohortOverlap(meds_overlap)
```

As well as generating these estimates for cohorts overall, we can also obtain stratified estimates. In this example we'll add age groups to our cohort table, and then obtain estimates stratified by these groups.
```{r}
cdm$meds <- cdm$meds |>
  PatientProfiles::addAge(ageGroup = list(c(0, 49), c(50, 150))) |>
  compute(temporary = FALSE, name = "meds") |>
  newCohortTable()
meds_overlap <- cdm$meds |>
  summariseCohortOverlap(strata = list("age_group"))
```

As with our overall results, we can quickly create tables and figures to view our stratified results.
```{r}
tableCohortOverlap(meds_overlap)
```

```{r}
plotCohortOverlap(meds_overlap,
  facet = "strata_level"
)
```

